%define releasebase 15
%define nbase cluster-admin-support
# allow --with[out] <feature> at rpm command line build
%{?_with_ohpc: %{expand: %%define as_ohpc 1}}
%{?_with_hadoop: %{expand: %%define as_hadoop 1}}
%{?_with_basement: %{expand: %%define as_basement 1}}
%{?_with_limulus: %{expand: %%define as_limulus 1}}
#pick the target machine
%if "1" == "%{as_basement}"
%define targetbase Basement
%else
%define targetbase Limulus
%endif
#pick the target cluster type
%if "1" == "%{as_hadoop}"
%define clusterbase hadoop
%else
%define clusterbase ohpc 
%endif

Name: %{nbase}-%{clusterbase}
Version:1.4
Group: %{targetbase}/Admin
Release: %{releasebase}.%{targetbase}
Summary: Utilities and Files for Clustered Systems
License: none
Group: System Environment/Base
Source: %{nbase}-%{version}.tgz
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}
%if "%{clusterbase}" == "ohpc"
Requires: whatsup pdsh-ohpc yad lm_sensors
%else
Requires: whatsup pdsh yad lm_sensors
%endif
Provides: gtemp
Vendor: Basement-Supercomputing.com

%description
Extra support scripts and configuration for Basement Supercomputer and Limulus
cluster administration. Includes checknodes.sh script (uses whats-up pingd) and 
sets cron.d file, ssh key generation for new users, includes Ganglia temperature 
module, shutdown script for nodes, limulus-node-temp.sh, /etc/fancontrol file for 
Limulus machines, AddUser and DelUser scripts (with gui mode).
Note: fancontrol.conf is for Limulus machines built by Basement Supercomputing. If you build
your own machine with different fans, makes sure you calibarate your fans!

%prep
%setup -q -n %{nbase}-%{version}

%build

%install
%if "%{clusterbase}" == "ohpc"
OPTPATH=ohpc
%else
OPTPATH=cluster
%endif

/bin/mkdir -p %{buildroot}/etc/warewulf/.%{targetbase}
cp  checknodes.sh  %{buildroot}/etc/warewulf/.%{targetbase}
chmod go-x %{buildroot}/etc/warewulf/.%{targetbase}/checknodes.sh
cp  gather.sh  %{buildroot}/etc/warewulf/.%{targetbase}
chmod go-x %{buildroot}/etc/warewulf/.%{targetbase}/gather.sh
%if "%{clusterbase}" == "ohpc"
cp  etc/slurm-node-up.sh etc/slurm-resume.sh etc/slurm-suspend.sh  %{buildroot}/etc/warewulf/.%{targetbase}
chmod go-x %{buildroot}/etc/warewulf/.%{targetbase}/slurm-*.sh
%endif 
cp  generate-ssh-keys.sh  %{buildroot}/etc/warewulf/.%{targetbase}

sed  --in-place 's/FILLINBASE/%{targetbase}/' %{buildroot}/etc/warewulf/.%{targetbase}/checknodes.sh
sed  --in-place 's/FILLINBASE/%{targetbase}/' %{buildroot}/etc/warewulf/.%{targetbase}/generate-ssh-keys.sh
# add profile.d so WCOLL get set on login
/bin/mkdir -p  %{buildroot}/etc/profile.d
cat <<EOF > %{buildroot}/etc/profile.d/pdsh.sh
if [ -e /etc/warewulf/.%{targetbase} ]; then
  export WCOLL=/etc/warewulf/.%{targetbase}/up-nodes
fi
EOF

cat <<EOF > %{buildroot}/etc/profile.d/pdsh.csh
if(  -e /etc/warewulf/.%{targetbase} ) then
setenv WCOLL /etc/warewulf/.%{targetbase}/up-nodes
endif
EOF
# add cron script to check nodes (uses whats-up)
/bin/mkdir -p %{buildroot}/etc/cron.d
cat <<EOF> %{buildroot}/etc/cron.d/pdsh-node-check
*/1  *  *  *  * root /etc/warewulf/.%{targetbase}/checknodes.sh &> /dev/null
EOF
# add Ganglia temperature module
mkdir -p  %{buildroot}/opt/$OPTPATH/admin/etc/ganglia
cp  gtemp.sh  %{buildroot}/opt/$OPTPATH/admin/etc/ganglia/gtemp.sh
chmod go-x %{buildroot}/opt/$OPTPATH/admin/etc/ganglia/gtemp.sh
# add 10/25 GbE config scripts
mkdir -p  %{buildroot}/opt/$OPTPATH/admin/etc
cp config-10G-ip %{buildroot}/opt/$OPTPATH/admin/etc
cp config-10G-ip-bridge %{buildroot}/opt/$OPTPATH/admin/etc

# set cron to report temps once each minute
/bin/mkdir -p %{buildroot}/etc/cron.d
cat <<EOF> %{buildroot}/etc/cron.d/report-ganglia-temp
*/1  *  *  *  * root sh /opt/$OPTPATH/admin/etc/ganglia/gtemp.sh &> /dev/null
EOF
# Add new GUI based AddUser, DelUser, NPstat
  /bin/mkdir -p %{buildroot}/usr/sbin
  /bin/cp sbin/AddUser %{buildroot}/usr/sbin
  /bin/cp sbin/DelUser %{buildroot}/usr/sbin
  /bin/cp sbin/NPstat %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/AddUser
  /bin/chmod 744 %{buildroot}/usr/sbin/DelUser
  /bin/chmod 744 %{buildroot}/usr/sbin/NPstat

%if "%{targetbase}" == "Limulus"
  # add node shutdown and fan control scripts for Limulus systems
  /bin/mkdir -p  %{buildroot}/etc/limfanctl/
  /bin/cp  etc/limfanctl/limfanctl.conf %{buildroot}/etc/limfanctl/limfanctl.conf
  
  # systemd shutdown nodes service
  install -Dp -m 0644 systemd/shutdown-nodes.service  %{buildroot}%{_unitdir}/shutdown-nodes.service
  
  /bin/mkdir -p  %{buildroot}/etc/warewulf/.Limulus
  /bin/cp  systemd/shutdown-nodes.sh %{buildroot}/etc/warewulf/.Limulus
  /bin/chmod 755 %{buildroot}/etc/warewulf/.Limulus/shutdown-nodes.sh

  # systemd limfanctl service
  install -Dp -m 0644 systemd/limfanctl.service  %{buildroot}%{_unitdir}/limfanctl.service

  /bin/mkdir -p %{buildroot}/usr/sbin
  /bin/cp sbin/limulus-node-temp %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/limulus-node-temp
  
  # install utilities  
  /bin/cp sbin/turbo-off  %{buildroot}/usr/sbin
  /bin/cp sbin/turbo-on %{buildroot}/usr/sbin
  /bin/cp sbin/turbo-status %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/turbo-*
  
  /bin/cp sbin/node-turbo-off  %{buildroot}/usr/sbin
  /bin/cp sbin/node-turbo-on %{buildroot}/usr/sbin
  /bin/cp sbin/node-turbo-status %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/node-turbo-*
  
  /bin/cp sbin/node-governor-set %{buildroot}/usr/sbin
  /bin/cp sbin/node-governor-status %{buildroot}/usr/sbin
  /bin/cp sbin/node-governor-avail %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/node-governor-*

  /bin/cp sbin/governor-set %{buildroot}/usr/sbin
  /bin/cp sbin/governor-status %{buildroot}/usr/sbin
  /bin/cp sbin/governor-avail %{buildroot}/usr/sbin
  /bin/chmod 744 %{buildroot}/usr/sbin/governor-*

# fancontrol files are selected on install because they vary by motherboard, so copy all
  /bin/mkdir -p %{buildroot}/etc/warewulf/.%{targetbase}/limfancontrol
  /bin/cp etc/fancontrol.* %{buildroot}/etc/warewulf/.%{targetbase}/limfancontrol
# include gnome menu options for tools (root only)
  /bin/mkdir -p  %{buildroot}/root/.local/share/applications/
  /bin/cp  gnome/*.desktop  %{buildroot}/root/.local/share/applications/
# include files used in post install 
  /bin/mkdir -p %{buildroot}/etc/warewulf/.%{targetbase}/post-install/%{clusterbase}
%if "%{clusterbase}" == "ohpc"
  /bin/cp  -r post-install/hpc/*  %{buildroot}/etc/warewulf/.%{targetbase}/post-install/%{clusterbase}
%else
  /bin/cp  -r post-install/hadoop/*  %{buildroot}/etc/warewulf/.%{targetbase}/post-install/%{clusterbase}
  /bin/mkdir -p  %{buildroot}/var/lib/tftpboot/pxelinux.cfg/images
  /bin/cp  var/vmlinuz  %{buildroot}/var/lib/tftpboot/pxelinux.cfg/images
  /bin/cp  var/initrd.img  %{buildroot}/var/lib/tftpboot/pxelinux.cfg/images
  /bin/cp  var/default.template %{buildroot}/var/lib/tftpboot/pxelinux.cfg 
  /bin/cp  opt/create-hdfs-disks.sh %{buildroot}/opt/cluster/admin/etc/.create-hdfs-disks.sh
%endif
  /bin/cp -r post-install/machines  %{buildroot}/etc/warewulf/.%{targetbase}/post-install/%{clusterbase}
%endif

%post
#enable services
systemctl enable shutdown-nodes.service
systemctl enable limfanctl.service
# provide base  file fancontrol
/bin/mkdir -p /var/warewulf/Limulus/
echo 25000 >/var/warewulf/Limulus/.do-not-remove-node-temperature-for-fancontrol
%if "%{clusterbase}" == "ohpc"
# If possible, change the slurm-{resume,suspend}.sh file to user slurm
# Needed because, cluster-admin-support does not require slurm install, but adds the
# Limulus suspend/resume files. The slurm user may not be defined until after slurm is installed.
if [ -e /etc/slurm/slurm.conf ]; then
  /bin/chown slurm:slurm  /etc/warewulf/.Limulus/slurm-suspend.sh
  /bin/chown slurm:slurm  /etc/warewulf/.Limulus/slurm-resume.sh
else
  echo "WARNING: No slurm user defined. slurm-{suspend,resume}.sh will have bad permissions." 
fi
%endif
# If Limulus add fancontrol and start fans
if [ -e %{_unitdir}/limfanctl.service ] ; then
  # Select correct fancontrol file
  MOTHERBOARD=`dmidecode |grep -A 4 "Base Board"|grep Product|sed 's/\tProduct Name: //'|sed 's/ /_/g'`
  case $MOTHERBOARD in
    GRYPHON_Z87)
      /bin/cp /etc/warewulf/.%{targetbase}/limfancontrol/fancontrol.GRYPHON_Z87 /etc/fancontrol
      ;;
    Z170M-PLUS|PRIME_Z270M-PLUS )
      /bin/cp /etc/warewulf/.%{targetbase}/limfancontrol/fancontrol.Z170M-PLUS  /etc/fancontrol      
      ;;
    PRIME_H370M-PLUS)
      /bin/cp /etc/warewulf/.%{targetbase}/limfancontrol/fancontrol.PRIME_H370M-PLUS  /etc/fancontrol      
      ;;
    PRIME_B450-PLUS)
      /bin/cp /etc/warewulf/.%{targetbase}/limfancontrol/fancontrol.PRIME_B450-PLUS  /etc/fancontrol      
      ;;
    *)
      echo "WARNING: Unsupported Limulus Motherboard. Node fancontrol will NOT work."
      echo "Contact Basement Supercomputing"
      ;;
  esac
fi
# add SSH keygen for new users
/bin/mkdir -p /etc/skel
# remove any previous additions
sed --in-place '/\/etc\/warewulf\/\.Limulus\/generate-ssh-keys\.sh/d' /etc/skel/.bashrc
sed --in-place '/\/etc\/warewulf\/\.Limulus\/generate-ssh-keys\.sh/d' /etc/skel/.cshrc

cat <<EOF>> /etc/skel/.bashrc
/etc/warewulf/.%{targetbase}/generate-ssh-keys.sh
EOF
cat <<EOF>>/etc/skel/.cshrc
/etc/warewulf/.%{targetbase}/generate-ssh-keys.sh
EOF

%clean
if [ $RPM_BUILD_ROOT != "/" ]; then
  rm -rf $RPM_BUILD_ROOT
fi
%preun
# stop/remove shutdown-nodes and limfanctl only on uninstall
if [ $1 = 0 ]; then
  systemctl --no-reload disable shutdown-nodes.service
  systemctl stop shutdown-nodes.service
  systemctl --no-reload disable limfanctl.service
  systemctl stop limfanctl.service
fi

%postun
if [ $1 = 0 ]; then
  # remove /etc/skel entries
  sed --in-place '/\/etc\/warewulf\/\.%{targetbase}\/generate-ssh-keys\.sh/d' /etc/skel/.cshrc
  sed --in-place '/\/etc\/warewulf\/\.%{targetbase}\/generate-ssh-keys\.sh/d' /etc/skel/.bashrc
  # remove the fancontrol temp files copied in post
  if [ -f /etc/fancontrol ]; then /bin/rm /etc/fancontrol;fi
  if [ -f /var/warewulf/Limulus/.do-not-remove-node-temperature-for-fancontrol ]; then /bin/rm /var/warewulf/Limulus/.do-not-remove-node-temperature-for-fancontrol;fi
fi

%files
%defattr(-,root,root)
/etc/profile.d
/etc/cron.d/pdsh-node-check
/etc/cron.d/report-ganglia-temp
/etc/warewulf/.%{targetbase}
%if "%{clusterbase}" == "ohpc"
/opt/ohpc/admin/etc/config-10G-ip
/opt/ohpc/admin/etc/config-10G-ip-bridge
/opt/ohpc/admin/etc/ganglia
%else
/opt/cluster/admin/etc/config-10G-ip
/opt/cluster/admin/etc/config-10G-ip-bridge
/opt/cluster/admin/etc/ganglia
/opt/cluster/admin/etc/.create-hdfs-disks.sh
/var/lib
%endif
%if "%{targetbase}" == "Limulus"
%{_unitdir}/shutdown-nodes.service
%{_unitdir}/limfanctl.service
/etc/limfanctl/limfanctl.conf
/usr/sbin/AddUser
/usr/sbin/DelUser
/usr/sbin/NPstat
/usr/sbin/limulus-node-temp
/usr/sbin/turbo-off
/usr/sbin/turbo-on
/usr/sbin/turbo-status
/usr/sbin/node-turbo-off
/usr/sbin/node-turbo-on
/usr/sbin/node-turbo-status
/usr/sbin/governor-set
/usr/sbin/governor-status
/usr/sbin/governor-avail
/usr/sbin/node-governor-set
/usr/sbin/node-governor-status
/usr/sbin/node-governor-avail
/root/.local/share/applications/
%endif

%changelog
* Mon Dec 21 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- added post install hadoop files for booting hadoop nodes for install 
* Sun Dec 13 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- added slurm-{resume, suspend}.sh in /etc/warewulf/.Limulus
* Mon Sep 21 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- added target option, ohpc or hadoop to allow /opt/{ohpc,cluster}/ option 
- default is ohpc
* Thu Sep 17 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- added post install file and machone defines to /etc/warewulf/.Limulus/post-install
* Wed Sep 02 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- small update to gtemp.sh so other apps (in addition to Ganglia) can get the CPU temp
* Tue Aug 18 2020 Douglas Eadline <deadline@basement-supercomputing.com>
- updated generate-ssh-keys.sh to use ecdsa -b 521 encryption
- added config-10G-ip, config-10G-ip-bridge to /opt/ohpc/admin/etc
* Tue May 5 2020  Douglas Eadline <deadline@basement-supercomputing.com>
- fixed AddUser and DelUser to perform "wwsh file resysnc" to push /etc/passwd to nodes
_ checks to see if wwsh is present.
* Thu Feb 20 2020  Douglas Eadline <deadline@basement-supercomputing.com>
- added commands to System Tools menu (root only) for AddUser, DelUser, NPstat
* Mon Feb 17 2020  Douglas Eadline <deadline@basement-supercomputing.com>
- Updated limululus-node-temp to check local fan locations in /sys/device
* Fri Jan 31 2020  Douglas Eadline <deadline@basement-supercomputing.com>
- Fixed bug in gtemp.sh when checking for k10temp-pci-00c3
- added "sh" to call /etc/cron.d/report-ganglia-temp, so it will work on nodes
* Tue Dec 31 2019  Douglas Eadline <deadline@basement-supercomputing.com>
- Added NPstat (node power utility) script 
* Fri Dec 06 2019  Douglas Eadline <deadline@basement-supercomputing.com>
- converted to systemd, changed any service calls to systemctl in scripts
- updates limulus-node-temp
- added fancontrol.PRIME_B450-PLUS
- updated gtem.sh for Ryzen temp module
* Thu Apr 25 2019  Douglas Eadline <deadline@basement-supercomputing.com>
- added AddUser and DelUser scripts
* Mon Jun 26 2017  Douglas Eadline <deadline@basement-supercomputing.com>
- updated fancontrol for PRIME_Z270M-PLUS motherboard
* Sun Jan 29 2017  Douglas Eadline <deadline@basement-supercomputing.com>
- To ensure nodes always shutdown, make pdsh in shutdown-nodes run as separate process, so it cannot freeze shutdown
* Tue Dec 20 2016 Douglas Eadline <deadline@basement-supercomputing.com>
- fixed pgrep in limulus-node-temp (use -f option with full path) 
* Wed Mar 23 2016 Douglas Eadline <deadline@basement-supercomputing.com>
- increase robustness of limfanctl by addeding sshd-check for nodes 
- and test to ensure lm_sensors is running on nodes in limulus-node-temp
* Mon Dec 21 2015 Douglas Eadline <deadline@basement-supercomputing.com>
- added auto setting for turbo mode in limulus-node-temp
- added /etc/limfanctl/limfanctl.conf functionality for limulus-node-temp
* Fri Dec 4 2015 Douglas Eadline <deadline@basement-supercomputing.com>
- added option for mulitple fancontrol files based on supported MBs
- added governor scripts, host turbo scrips
* Fri Feb 20 2015 Douglas Eadline <deadline@basement-supercomputing.com>
- added Limulus node-turbo-off/on/status scripts
* Wed Jan 28 2015  Douglas Eadline <deadline@basement-supercomputing.com>
- added Limulus fancontrol files (limfanctl, fancontrol, limulus-node-temps)
* Sun Jan 06 2013  Douglas Eadline <deadline@basement-supercomputing.com>
- added report-ganglia-temp to cron.d to collect temp (copied to nodes throught WW)
- added shutdown-nodes init script to ensure nodes with HDD shutdown properly
* Tue Jul 24 2012  Douglas Eadline <deadline@basement-supercomputing.com>
- created
